#
# Copyright (c) 2015 Midokura SARL, All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
import functools

from charmhelpers.contrib.openstack import context
from charmhelpers.contrib.openstack import templating
from charmhelpers.contrib.openstack import utils
from charmhelpers.core import hookenv
from charmhelpers.core import host
from charmhelpers.core.services import base
from charmhelpers.core.services import helpers
from midonet_helpers import puppet
from midonet_helpers import relations as common_relations
from midonet_helpers import repositories
import relations

TEMPLATES = 'templates/'


class Config(dict):
    def __init__(self, *args):
        self['config'] = hookenv.config()


def register_ost_configs(amqp_ctxt, release=None):
    release = release or utils.os_release('neutron-common')
    configs = templating.OSConfigRenderer(templates_dir=TEMPLATES,
                                          openstack_release=release)
    configs.register('/etc/neutron/neutron.conf', [amqp_ctxt])
    return configs


def manage():
    nova = relations.NovaRelation()
    config = Config()
    hookenv.status_set('maintenance', 'reconfiguring charm')
    ost_origin = config['config'].get('openstack-origin')
    midonet_origin = config['config'].get('midonet-origin')
    if None in (ost_origin, midonet_origin):
        hookenv.status_set('maintenance', 'Lacking repository configuration')
        return
    ost_release = utils.get_os_codename_install_source(ost_origin)
    try:
        repo = repositories.config('%s/%s' % (ost_release, midonet_origin),
                                   config['config'].get('mem-username'),
                                   config['config'].get('mem-password'))
    except ValueError as ver:
        hookenv.status_set('maintenance',
                           'Wrong repository configuration: {}'.format(ver))
        return

    amqp = context.AMQPContext()
    ost_cfgs = register_ost_configs(amqp)
    midonet = common_relations.MidonetApiRelation()

    manager = base.ServiceManager([
        {
            'service': 'neutron-dhcp-agent',
            'required_data': [repo,
                              nova,
                              midonet,
                              amqp()],
            'data_ready': [
                helpers.render_template(
                    source='common.yaml',
                    target='/var/lib/hiera/neutron_agents/common.yaml'),
                helpers.render_template(
                    source='dhcp_agent.ini',
                    target='/etc/neutron/dhcp_agent.ini'),
                puppet.InstallPackagesCallback(
                    ('python-neutron-plugin-midonet',)),
                functools.partial(ost_cfgs.write_all),
            ],
        },
        {
            'service': 'neutron-metadata-agent',
            'required_data': [config,
                              nova],
            'data_ready': [
                helpers.render_template(
                    source='metadata_agent.ini',
                    target='/etc/neutron/metadata_agent.ini'),
            ],
        },
    ])
    manager.manage()
    if (host.service_running('neutron-dhcp-agent') and
            host.service_running('neutron-metadata-agent')):
        hookenv.status_set('active', 'dhcp and metadata agents up and running')
